// Sample app that uses COM interop to call the
// ICorPublish family of interfaces for enumerating
// managed processes and appdomains.  See corpub.cs
// to take a look at the interop shims that are being
// used.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;

class App
{
	static void Main(string[] args)
	{
        DumpProcesses();
	}

    static void DumpProcesses()
    {
        // Get an enumerator for managed processes only.
        //
        ICorPublish corPub = (ICorPublish)new CorpubPublish();
        ICorPublishProcessEnum corPubEnum =
            corPub.EnumProcesses(COR_PUB_ENUMPROCESS.COR_PUB_MANAGEDONLY);

        // Run through each process, showing a few things about it.
        //
        uint NumProcesses = corPubEnum.GetCount();
        ICorPublishProcess[] procs = new ICorPublishProcess[NumProcesses];
        uint cFetched = 0;
        corPubEnum.Next(NumProcesses, procs, out cFetched);

        Debug.Assert(cFetched == NumProcesses);

        foreach( ICorPublishProcess proc in procs )
        {
            DumpProcess(proc);
        }
    }

    static void DumpProcess( ICorPublishProcess proc )
    {
        // Show some process information...
        //
        Console.WriteLine("PID:         {0}", proc.GetProcessID());
        Console.WriteLine("IsManaged:   {0}", proc.IsManaged());

        char[] DisplayNameChars = new char[256];
        uint cch;
        proc.GetDisplayName((uint)DisplayNameChars.Length, out cch, DisplayNameChars);
        string DisplayName = new string(DisplayNameChars, 0, (int)cch);

        Console.WriteLine("DisplayName: {0}", DisplayName);

        // Get an enumerator for the appdomains inside this process
        // and then run through them, showing some stats on them...
        //
        ICorPublishAppDomainEnum enumAppDomains = proc.EnumAppDomains();
        uint NumDomains = enumAppDomains.GetCount();
        ICorPublishAppDomain[] AppDomains = new ICorPublishAppDomain[NumDomains];
        uint cFetched = 0;
        enumAppDomains.Next(NumDomains, AppDomains, out cFetched);
        Debug.Assert(cFetched == NumDomains);

        foreach( ICorPublishAppDomain ad in AppDomains )
        {
            ad.GetName((uint)DisplayNameChars.Length, out cch, DisplayNameChars);
            DisplayName = new string(DisplayNameChars, 0, (int)cch);
            Console.WriteLine("AppDomain {0}: {1}", ad.GetID(), DisplayName);
        }
    }
}
