// corpub.cs
//
// This file defines some COM interop shims to CLSID_CorpublishProcess
// its associated interfaces for enumerating managed/unmanaged processes
// and appdomains within managed processes as defined in corpub.idl.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Text;
using System.Runtime.InteropServices;

[ComImport, Guid("047a9a40-657e-11d3-8d5b-00104b35e7ef")]
public class CorpubPublish
{
    // [default] interface  ICorPublish;
    // interface            ICorPublishProcess;
    // interface            ICorPublishAppDomain;
    // interface            ICorPublishProcessEnum;
    // interface            ICorPublishAppDomainEnum;
}

public enum COR_PUB_ENUMPROCESS
{
    COR_PUB_MANAGEDONLY = 0x00000001
}

[
  Guid("9613A0E7-5A68-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublish
{
    /*
     * This method is used to retrieve a list of processes on a given
     * machine.  The list of processes may or may not be managed
     * depending on the filter flags given.  The list is based on a
     * snapshot of the processes running when the enum method is
     * called.  If other processes start or existing ones stop while
     * enumerating, the list will remain the same.
     */
    ICorPublishProcessEnum EnumProcesses( COR_PUB_ENUMPROCESS ProcessType );
    ICorPublishProcess GetProcess( uint pid );
}

[
  Guid("C0B22967-5A69-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublishEnum
{
    void Skip( uint celt );
    void Reset();
    ICorPublishEnum Clone();
    uint GetCount();
}

[
  Guid("18D87AF1-5A6A-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublishProcess
{
    /*
     * Returns true if the process is known to have managed code
     * running in it.  Note that this could return false now but true
     * later in the case where managed code was loaded into the
     * process after the enum was built.
     */
    [return: MarshalAs(UnmanagedType.Bool)] bool IsManaged();
    
    /*
     * Enumerate the list of known application domains in the target process.
     */
    ICorPublishAppDomainEnum EnumAppDomains();
    
    /*
     * Returns the OS ID for the process in question.
     */
    uint GetProcessID();
    
    /*
     * Get the display name for a process.
     */
    [PreserveSig]
    void GetDisplayName( uint cchNameMax,
                         [Out] out uint cchName,
                         [Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex=1)] char[] Name );
}


[
  Guid("D6315C8F-5A6A-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublishAppDomain
{
    /*
     * Get the name and ID for an application domain.
     */
    uint GetID();
    
    /*
     * Get the name for an application domain.
     */
    [PreserveSig]
    void GetName( uint cchNameMax, 
                  [Out] out uint cchName,
                  [Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex=1)] char[] Name );
}

[
  Guid("A37FBD41-5A69-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublishProcessEnum
{
    void Skip( uint celt );
    void Reset();
    ICorPublishEnum Clone();
    uint GetCount();

    [PreserveSig]
    void Next( uint celt,
			   [Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex=2)] ICorPublishProcess[] objects,
			   [Out] out uint celtFetched );
}

[
  Guid("9F0C98F5-5A6A-11d3-8F84-00A0C9B4D50C"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface ICorPublishAppDomainEnum
{
    void Skip( uint celt );
    void Reset();
    ICorPublishEnum Clone();
    uint GetCount();

    [PreserveSig]
    void Next( uint celt,
               [Out, MarshalAs(UnmanagedType.LPArray, SizeParamIndex=2)] ICorPublishAppDomain[] objects,
               [Out] out uint celtFetched );
}

